library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


entity Antirebotes is 
    Port (
            clk             : in STD_LOGIC; 
            reset           : in std_logic; 
            boton           : in STD_LOGIC;
            filtrado        : out STD_LOGIC
            );
end Antirebotes;

architecture behavioral of Antirebotes is

    --Sincronizador-- 
    signal Q1: std_logic;
    signal Q2: std_logic;

    -- detector de flancos-- 
    signal flanco: std_logic; 
    signal Q3: std_logic;


    -- declaracion de FSM ----
    type state_t is (S_NADA, S_BOTON); 
    signal state : state_t;

    -- Temporizador--
    constant MAX        : integer :=(125*10**3); --**6 en weblab
    signal E            : std_logic;     --seal de habilitacin del temporizador
    signal T            : std_logic;     --seal que ha llegado el valor maximo en la temporizacion
    signal cnt_1ms      : integer range 0 to MAX-1;

begin

    --Sincronizador + FF Detector de flancos (FF biestables D en cadena)
    process (clk, reset)
    begin
        if (reset = '1') then 
            Q1<='0';
            Q2<='0';
            Q3<='0';
        elsif (clk'event and clk ='1') then --CORREGIR 
            Q1 <= boton;
            Q2 <= q1;
            Q3 <= q2;
                        
        end if;
    end process;


--    --Sincronizador 
--    process(clk,reset) 
--        begin
--            if (reset = '1') then 
--                Q1 <= '0';
--            elsif (clk'event and clk ='1') then 
--                if (boton ='1') then
--                    Q1 <= '1';
--                elsif (boton='0') then
--                    Q1<='0';
--                end if; 
--            end if;
 
--    end process; 
    
--    process(clk,reset) 
--    begin
--        if (reset = '1') then 
--            Q2 <= '0';
--        elsif (clk'event and clk ='1') then 
--            if (Q1 ='1') then
--                Q2 <='1';
--            elsif (Q1='0') then 
--                Q2 <='0';
--            end if;
--        end if;
--    end process;
    
    
--    --DETECTOR DE FLANCOS
--    process(clk,reset) 
--    begin
--        if (reset = '1') then
--            Q3 <= '0';
--        elsif (clk'event and clk ='1') then 
--            if (Q2 ='1') then
--                Q3<='1';
--            elsif (Q2='0')	then 
--                Q3<='0';
--            end if;
--        end if;
--    end process;
    
    
    --Negar Q3 y puerta AND
    flanco<= not (Q3) and Q2;
    
    
    
    ---Implementacion de FSM explicita-- (ecuaciones de estado) 
    process(clk,reset)
    begin
        if (reset = '1') then 
            state <= S_NADA;
        elsif (clk'event and clk ='1') then 
            case state is
                when S_NADA => 
                    if flanco = '1' then
                        state <= S_BOTON; 
                    end if;    
                when S_BOTON => 
                    if T = '1' then
                        state <=S_NADA; 
                    end if;
            end case; 
         end if;
    end process;
    
    
    -- ecuaciones de salida
    
    filtrado <= '1' when state = S_BOTON and Q2 = '1' and T='1' else '0'; -- Mealy, T es 1 en la transicion de estados
    E <= '1' when state = S_BOTON else '0';	-- Moore
    
    
    --Temporizador, contador de 1ms. Es como un divisor de frecuencia
    process(clk,reset) 
    begin
        if (reset='1') then 
            cnt_1ms<= 0;
        elsif (clk'event and clk ='1') then 
            if (E='1') then
                if (cnt_1ms<MAX-1) then  
                    cnt_1ms <= cnt_1ms+1;
                else 
                    cnt_1ms <= 0; 
                end if;
            end if; 
         end if;
    end process;
    
    T<='1' when (cnt_1ms=MAX-2) and E='1' else '0'; 
    
    
end behavioral;